from shopdatabase import DatabaseController
from parser import Parser
import sqlite3

ps = Parser()
dbpath = ps.get_db_path()

class CheckController(DatabaseController):
    def __init__(self):
        DatabaseController.__init__(self)

    def check_in(self, username, barcode):
        """
        Given a barcode and its corresponding DVD will be re-shelved into the
        database, the corresponding borrower's borrowed quantity is reduced by
        1.

        Returns True on success check-in, False means the dvd is not yet lent
        out.
        """
        if self.check_dvd_state(barcode, 0):
            return False

        if not self.check_dvd_borrower(barcode) == username.lower():
            return False

        conn = sqlite3.connect(dbpath)
        c = conn.cursor()
        c.execute("UPDATE dvdlist SET out=0 WHERE barcode=(?)", (barcode,))
        c.execute("UPDATE userdb SET borrowed=borrowed-1 WHERE username=(?)",
                (username.lower(),))
        conn.commit()
        conn.close()
        return True

    def check_dvd_borrower(self, barcode):
        """
        This checks the borrower of a specific DVD, it returns the username of
        the borrower
        """
        conn = sqlite3.connect(dbpath)
        c = conn.cursor()
        raw = c.execute("SELECT borrower FROM dvdlist WHERE barcode=(?)",
                (barcode,))
        try:
            data = raw.fetchone()
            name = data[0][0]
        except TypeError:
            conn.close()
            return None
        except StopIteration:
            conn.close()
            return None
        else:
            conn.close()
            return data[0].lower()

    def check_dvd_state(self, barcode, state):
        """
        Check whether a specific dvd is under a correct state.
        Return True or False on yes or no.
        """
        conn = sqlite3.connect(dbpath)
        c = conn.cursor()
        rawdata = c.execute("SELECT * FROM dvdlist WHERE barcode=(?)",
                (barcode,))
        item = rawdata.fetchone()
        conn.close()
        try:
            item[3] == state
        except TypeError:
            return False
        else:
            return item[3] == state

    def check_out(self, username, barcode):
        """
        Given a barcode and a username we can assign the DVD to the customer.

        Return True on success, False on DVD unavailable.
        """
        if not self.check_dvd_state(barcode, 0):
            return False

        if self.check_user_quantity(username.lower()) == 6:
            return False

        conn = sqlite3.connect(dbpath)
        c = conn.cursor()
        c.execute("UPDATE dvdlist SET out=1, borrower=(?) WHERE barcode=(?)",
                (username.lower(), barcode))
        c.execute("UPDATE userdb SET borrowed = borrowed + 1 \
                WHERE username=(?)", (username.lower(),))
        conn.commit()
        conn.close()
        return True
    
    def check_user_quantity(self, username):
        """
        It returns on a user's borrowed quantity.
        """
        conn = sqlite3.connect(dbpath)
        c = conn.cursor()
        raw = c.execute("SELECT * FROM userdb WHERE username=(?)",
                (username.lower(),))
        info = raw.fetchone()
        conn.close()
        try:
            info[5]
        except TypeError:
            return False
        else:
            return info[5]
