import gtk

from stockcontroller import StockController
from parser import Parser
from logger import Logger
from checkcontroller import CheckController

parse = Parser()
sc = StockController()
log = Logger()
cc = CheckController()

profit_rate = parse.get_profit_rate()
deposit_rate = parse.get_deposit_rate()

class CheckOutUI(gtk.Window):
    """
    Check Out UI will consists of a line of notification, 6 lines of barcode and
    its corresponding prices and titles.
    """
    def __init__(self, customer, operator):
        self.customer = customer
        self.operator = operator
        self.totalPrice = 0
        self.barcodes = []
        gtk.Window.__init__(self)
        self.set_resizable(False)
        self.set_title("Check Out Interface")
        self.set_position(gtk.WIN_POS_CENTER)

        # Layout
        table = gtk.Table()
        customerLabel = gtk.Label(str("For customer " + self.customer))
        leftLabel = gtk.Label(str(6 -
                              int(cc.check_user_quantity(self.customer)) +
                              " left."))
        barcodeLabel = gtk.Label("Barcode")
        titleLabel = gtk.Label("Title")
        priceLabel = gtk.Label("Price")


        # First Item
        self.firstEntry = gtk.Entry()
        self.firstTitleEntry = gtk.Entry()
        self.firstTitleEntry.set_editable(False)
        self.firstPriceEntry = gtk.Entry()
        self.firstPriceEntry.set_editable(False)

        # Second Item
        self.secondEntry = gtk.Entry()
        self.secondTitleEntry = gtk.Entry()
        self.secondTitleEntry.set_editable(False)
        self.secondPriceEntry = gtk.Entry()
        self.secondPriceEntry.set_editable(False)

        # Third Item
        self.thirdEntry = gtk.Entry()
        self.thirdTitleEntry = gtk.Entry()
        self.thirdTitleEntry.set_editable(False)
        self.thirdPriceEntry = gtk.Entry()
        self.thirdPriceEntry.set_editable(False)

        # Fourth Item
        self.fourthEntry = gtk.Entry()
        self.fourthTitleEntry = gtk.Entry()
        self.fourthTitleEntry.set_editable(False)
        self.fourthPriceEntry = gtk.Entry()
        self.fourthPriceEntry.set_editable(False)

        # Fifth Item
        self.fifthEntry = gtk.Entry()
        self.fifthTitleEntry = gtk.Entry()
        self.fifthTitleEntry.set_editable(False)
        self.fifthPriceEntry = gtk.Entry()
        self.fifthPriceEntry.set_editable(False)

        # Sixth Item
        self.sixthEntry = gtk.Entry()
        self.sixthTitleEntry = gtk.Entry()
        self.sixthTitleEntry.set_editable(False)
        self.sixthPriceEntry = gtk.Entry()
        self.sixthPriceEntry.set_editable(False)

        # Buttons
        checkPriceButton = gtk.Button("Check Price")
        checkPriceButton.set_size_request(70,30)

        checkOutButton = gtk.Button("Check Out")
        checkOutButton.set_size_request(70, 30)

        quitButton = gtk.Button("Cancel")
        quitButton.set_size_request(70, 30)

        # Total label
        self.totalLabel = gtk.Label("  ")

        # Add elements to layout

        table.attach(gtk.Label("   "), 0, 2, 0, 1)
        table.attach(customerLabel, 2, 4, 0, 1)
        table.attach(leftLabel, 4, 6, 0, 1)
    
        table.attach(barcodeLabel, 0, 2, 1, 2)
        table.attach(titleLabel, 2, 4, 1, 2)
        table.attach(priceLabel, 4, 6, 1, 2)
        
        table.attach(self.firstEntry, 0, 2, 2, 3)
        table.attach(self.firstTitleEntry, 2, 4, 2, 3)
        table.attach(self.firstPriceEntry, 4, 6, 2, 3)

        table.attach(self.secondEntry, 0, 2, 3, 4)
        table.attach(self.secondTitleEntry, 2, 4, 3, 4)
        table.attach(self.secondPriceEntry, 4, 6, 3, 4)

        table.attach(self.thirdEntry, 0, 2, 4, 5)
        table.attach(self.thirdTitleEntry, 2, 4, 4, 5)
        table.attach(self.thirdPriceEntry, 4, 6, 4, 5)
        
        table.attach(self.fourthEntry, 0, 2, 5, 6)
        table.attach(self.fourthTitleEntry, 2, 4, 5, 6)
        table.attach(self.fourthPriceEntry, 4, 6, 5, 6)
        
        table.attach(self.fifthEntry, 0, 2, 6, 7)
        table.attach(self.fifthTitleEntry, 2, 4, 6, 7)
        table.attach(self.fifthPriceEntry, 4, 6, 6, 7)

        table.attach(self.sixthEntry, 0, 2, 7, 8)
        table.attach(self.sixthTitleEntry, 2, 4, 7, 8)
        table.attach(self.sixthPriceEntry, 4, 6, 7, 8)

        table.attach(self.totalLabel, 0, 2, 8, 9)
        table.attach(gtk.Label("                "), 2, 3, 8, 9)
        table.attach(checkPriceButton, 3, 4, 8, 9)
        table.attach(checkOutButton, 4, 5, 8, 9)
        table.attach(quitButton, 5, 6, 8, 9)

        self.add(table)

        # signals
        checkPriceButton.connect("clicked", self.fetch_price)
        checkOutButton.connect("clicked", self.check_out)
        quitButton.connect("clicked", self.on_quit)

        self.show_all()

    def on_quit(self, widget):
        from cashier import Cashier
        CashierUI(self.operator)
        self.destroy()

    def get_dvd_price(self, barcode):
        """
        Fetch requested DVD price
        """
        info = sc.get_dvd_info(barcode)
        if info is None:
            pass
        else:
            return info[5] * profit_rate

    def get_dvd_title(self, barcode):
        """
        Fetch requested DVD title
        """
        info = sc.get_dvd_info(barcode)
        if info is None:
            pass
        else:
            return info[0]

    def fetch_price(self, *args):
        """
        Fetch all prices if any text entry has something to extract information
        from.
        """
        self.barcodes = []
        barcode1 = self.firstEntry.get_text()
        barcode2 = self.secondEntry.get_text()
        barcode3 = self.thirdEntry.get_text()
        barcode4 = self.fourthEntry.get_text()
        barcode5 = self.fifthEntry.get_text()
        barcode6 = self.sixthEntry.get_text()

        if barcode1 is not "":
            price = self.get_dvd_price(barcode1)
            title = self.get_dvd_title(barcode1)
            self.firstPriceEntry.set_text(str(price * profit_rate +
                    self.get_deposit(barcode1)))
            self.firstTitleEntry.set_text(title)
            self.barcodes.append((barcode1, self.firstPriceEntry.get_text()))
        else:
            self.firstPriceEntry.set_text("")
            self.firstTitleEntry.set_text("")

        if barcode2 is not "":
            price = self.get_dvd_price(barcode2)
            title = self.get_dvd_title(barcode2)
            self.secondPriceEntry.set_text(str(price * profit_rate +
                    self.get_deposit(barcode2)))
            self.secondTitleEntry.set_text(title)
            self.barcodes.append((barcode2, self.secondPriceEntry.get_text()))
        else:
            self.secondPriceEntry.set_text("")
            self.secondTitleEntry.set_text("")

        if barcode3 is not "":
            price = self.get_dvd_price(barcode3)
            title = self.get_dvd_title(barcode3)
            self.thirdPriceEntry.set_text(str(price * profit_rate +
                    self.get_deposit(barcode3)))
            self.thirdTitleEntry.set_text(title)
            self.barcodes.append((barcode3, self.thirdPriceEntry.get_text()))
        else:
            self.thirdPriceEntry.set_text("")
            self.thirdTitleEntry.set_text("")

        if barcode4 is not "":
            price = self.get_dvd_price(barcode4)
            title = self.get_dvd_title(barcode4)
            self.fourthPriceEntry.set_text(str(price * profit_rate +
                    self.get_deposit(barcode4)))
            self.fourthTitleEntry.set_text(title)
            self.barcodes.append((barcode4, self.fourthPriceEntry.get_text()))
        else:
            self.fourthPriceEntry.set_text("")
            self.fourthTitleEntry.set_text("")

        if barcode5 is not "":
            price = self.get_dvd_price(barcode5)
            title = self.get_dvd_title(barcode5)
            self.fifthPriceEntry.set_text(str(price * profit_rate +
                    self.get_deposit(barcode5)))
            self.fifthTitleEntry.set_text(title)
            self.barcodes.append((barcode5, self.fifthPriceEntry.get_text()))
        else:
            self.fifthPriceEntry.set_text("")
            self.fifthTitleEntry.set_text("")

        if barcode6 is not "":
            price = self.get_dvd_price(barcode6)
            title = self.get_dvd_title(barcode6)
            self.sixthPriceEntry.set_text(str(price * profit_rate +
                    self.get_deposit(barcode6)))
            self.sixthTitleEntry.set_text(title)
            self.barcodes.append((barcode6, self.sixthPriceEntry.get_text()))
        else:
            self.sixthPriceEntry.set_text("")
            self.sixthTitleEntry.set_text("")

        self.calculate_total()

    def calculate_total(self):
        from decimal import Decimal
        total = 0
        price1 = self.firstPriceEntry.get_text()
        price2 = self.secondPriceEntry.get_text()
        price3 = self.thirdPriceEntry.get_text()
        price4 = self.fourthPriceEntry.get_text()
        price5 = self.fifthPriceEntry.get_text()
        price6 = self.sixthPriceEntry.get_text()

        if price1 is not "":
            total += Decimal(price1)
        if price2 is not "":
            total += Decimal(price2)
        if price3 is not "":
            total += Decimal(price3)
        if price4 is not "":
            total += Decimal(price4)
        if price5 is not "":
            total += Decimal(price5)
        if price6 is not "":
            total += Decimal(price6)

        self.totalPrice = total
        self.totalLabel.set_text("Total is: " + str(total))

    def get_deposit(self, barcode):
        info = sc.get_dvd_info(barcode)
        if info is None:
            pass
        else:
            return info[5] * deposit_rate

    def check_out(self, widget):
        self.fetch_price()
        borrowingQuan = len(self.barcodes)
        userQuan = cc.check_user_quantity(self.customer)
        for num in range(borrowingQuan):
            if userQuan + borrowingQuan >= 6:
                dialog = gtk.Dialog("Error!", self, gtk.DIALOG_MODAL, 
                        (gtk.STOCK_OK, gtk.RESPONSE_ACCEPT |
                        gtk.DIALOG_DESTROY_WITH_PARENT))

                err_msg = gtk.Label(str("Customer has " + userQuan + 
                            "DVDs already."))
                dialog.vbox.pack_start(err_msg)
                err_msg.show()
                dialog.run()
                dialog.destroy()
                return None
            else:
                ret = cc.check_out(self.customer, self.barcodes[num][0])
                if ret == True:
                    log.log_check_out(self.customer, self.barcodes[num][0],
                            self.operator, self.barcodes[num][1])
                    
                else:
                    dialog = gtk.Dialog("Error!", self, gtk.DIALOG_MODAL, 
                        (gtk.STOCK_OK, gtk.RESPONSE_ACCEPT |
                        gtk.DIALOG_DESTROY_WITH_PARENT))

                    err_msg = gtk.Label(str(self.barcodes[num][0] + 
                            " is already borrowed."))
                    dialog.vbox.pack_start(err_msg)
                    err_msg.show()
                    dialog.run()
                    dialog.destroy()
                    return None

        dialog = gtk.Dialog("Success!", self, gtk.DIALOG_MODAL, 
                (gtk.STOCK_OK, gtk.RESPONSE_ACCEPT |
                gtk.DIALOG_DESTROY_WITH_PARENT))

        err_msg = gtk.Label("Successfully borrowed.")
        dialog.vbox.pack_start(err_msg)
        err_msg.show()
        dialog.run()
        dialog.destroy()


class CheckInUI(gtk.Window):
    def __init__(self, customer, operator):
        gtk.Window.__init__(self)
        self.customer = customer
        self.operator = operator
        self.set_position(gtk.WIN_POS_CENTER)
        self.set_title("Check In Interface")
        self.set_resizable(False)

        # Layout
        vbox = gtk.VBox()
        entryHBox = gtk.HBox()
        buttonHBox = gtk.HBox()
        topHBox = gtk.HBox()

        # Widgets
        self.customerLabel = gtk.Label(str("For customer: " + self.customer))
        returnLabel = gtk.Label("  Barcode:   ")
        self.barcodeEntry = gtk.Entry()
        checkInButton = gtk.Button("Check In")
        cancelButton = gtk.Button("Cancel")
        self.fundreturnLabel = gtk.Label("")

        # Add widgets to hboxs
        topHBox.add(self.customerLabel)
        entryHBox.add(returnLabel)
        entryHBox.add(self.barcodeEntry)
        buttonHBox.add(self.fundreturnLabel)
        buttonHBox.add(checkInButton)
        buttonHBox.add(cancelButton)

        vbox.add(topHBox)
        vbox.add(entryHBox)
        vbox.add(buttonHBox)
        self.add(vbox)

        # Signals
        self.barcodeEntry.connect("activate", self.check_in)
        checkInButton.connect("clicked", self.check_in)
        cancelButton.connect("clicked", self.on_quit)
        self.connect("destroy", self.on_quit)

        self.show_all()

    def on_quit(self, widget):
        self.destroy()
        from cashier import Cashier
        Cashier(self.operator)

    def check_in(self, widget):
        barcode = self.barcodeEntry.get_text()
        ret = cc.check_in(self.customer, barcode)
        deposit = parse.get_deposit_rate() * sc.get_dvd_info(barcode,
                "price")[0]

        if ret == False:
            dialog = gtk.Dialog("Error!", self, gtk.DIALOG_MODAL,
                    (gtk.STOCK_OK, gtk.RESPONSE_ACCEPT |
                    gtk.DIALOG_DESTROY_WITH_PARENT))
            reg_suc = gtk.Label("Check in Failure.")
            dialog.vbox.pack_start(reg_suc)
            reg_suc.show()
            dialog.run()
            dialog.destroy()
        else:
            log.log_check_in(self.barcodeEntry.get_text(), self.operator)
            dialog = gtk.Dialog("Success", self, gtk.DIALOG_MODAL,
                    (gtk.STOCK_OK, gtk.RESPONSE_ACCEPT |
                    gtk.DIALOG_DESTROY_WITH_PARENT))
            reg_suc = gtk.Label("Deposit return: " + str(deposit))
            dialog.vbox.pack_start(reg_suc)
            reg_suc.show()
            dialog.run()
            dialog.destroy()
            self.barcodeEntry.set_text("")
