import time
import os
import os.path

trans_log = "logs/transaction.log"
admin_log = "logs/administration.log"

class Logger(object):
    """
    The Logger will record every activity that happens within the software.

    Methods are listed here:
        log_add_user(username [, database [, operator]]) 
            Defaults are "userdb" and "None"
        log_check_in(barcode)
        log_check_out(username, barcode, operator, money)
        log_change_user_data(username [, operator])
        log_change_staff_data(username [, operator])
    """
    def __init__(self):
        """
        Checks if the directory 'logs' exists under current directory when
        program is initialized.
        """
        if not os.path.exists("logs"):
            os.mkdir("logs")
        else:
            pass # for if statement completeness

    def __current_time(self):
        return time.strftime("%Y%m%d%H%M")

    def __write(self, log, data):
        """
        Writes the data to whatever the log passed into.
        """
        f = open(log, "a")
        f.write(data)
        f.close()

    def log_add_user(self, username, database, operator, flag):
        """
        Records self registering event.

        Default value for database is userdb, Operator is none customer self 
        register.
        """
        if flag is 0:
            group = "customer"
        elif flag is 1:
            group = "staff"
        operation_time = self.__current_time()
        data = operation_time + " " + username + " is added to " + database + \
                "\n"
        self.__write(admin_log, data)

    def log_check_in(self, barcode, operator):
        """
        This records the exact time of check in of any DVD.
        """
        operation_time = self.__current_time()
        data = operation_time + " " + barcode + " is returned by " + operator \
                + ".\n"
        self.__write(trans_log, data)

    def log_check_out(self, username, barcode, operator, money):
        """
        This records the log of check-out events and its information related to
        check-out such as which customer, DVD barcode, by which operator and
        how much did the customer pay.
        """
        operation_time = self.__current_time()
        data = operation_time + " " + username + " has checked out " + barcode\
                + " by operator " + operator + " fee is " + money + "\n"
        self.__write(trans_log, data)

    def log_change_user_data(self, username, operator="customer himself"):
        """
        This records the change on a user's data, normally will be done by the
        user himself.

        The detailed content is not recorded, it just mentions when who changed
        info about himself.
        """
        operation_time = self.__current_time()
        data = operation_time + " " + username + " changed info by " + \
                operator + "\n"
        self.__write(admin_log, data)

    def log_change_staff_data(self, username, operator="sysadmin"):
        """
        When a system administrator adds a staff, this will be recorded down
        too.
        """
        operation_time = self.__current_time()
        data = operation_time + " " + username + "'s info has been changed by \
        " + operator + "\n"
        self.__write(admin_log, data)
