#coding=utf-8
#
# File:     loginwindow.py
# Author:   Luxing Huang 100113287
# Date:     2014/03/12
# Purpose:  It's the general login face
#

import gtk

class LoginWindow(gtk.Window):
    def __init__(self, *args):
        self.staff_login = False
        gtk.Window.__init__(self)
        self.set_position(gtk.WIN_POS_CENTER)
        self.connect("destroy", self.do_quit)
        self.set_resizable(False)

        from modules.parser import Parser
        conf_parser = Parser()
        shoptitle = conf_parser.get_shop_name()
        self.set_title(shoptitle)

        # Top hbox layer
        self.loginLabel = gtk.Label("  Username:  ")
        self.usernameInput = gtk.Entry()

        self.passwdLabel = gtk.Label("  Password:  ")
        self.passwdInput = gtk.Entry()
        self.passwdInput.set_visibility(False)

        # Bottom hbox layer
        self.checkbox = gtk.CheckButton("Staff?")
        self.loginButton = gtk.Button("Login")
        self.loginButton.set_size_request(70, 30)
        self.registerButton = gtk.Button("Register")
        self.registerButton.set_size_request(70, 30)
        self.cancelButton = gtk.Button("Cancel")
        self.cancelButton.set_size_request(70, 30)

        #Signals
        self.checkbox.connect("toggled", self.on_checkButtonToggled)
        self.passwdInput.connect("activate", self.do_login)
        self.loginButton.connect("clicked", self.do_login)
        self.registerButton.connect("clicked", self.do_customer_register)
        self.cancelButton.connect("clicked", self.do_quit)

        #Layout
        self.table = gtk.Table()

        #Add elements to layout

        self.table.attach(self.loginLabel, 0, 2, 0, 1)
        self.table.attach(self.usernameInput, 2, 4, 0, 1)
        self.table.attach(self.passwdLabel, 4, 7, 0, 1)
        self.table.attach(self.passwdInput, 7, 9, 0, 1)
        self.table.attach(gtk.Label(), 0, 9, 1, 2)
        self.table.attach(self.checkbox, 0, 2, 2, 3)
        self.table.attach(self.loginButton, 6, 7, 2, 3)
        self.table.attach(self.registerButton , 7, 8, 2, 3)
        self.table.attach(self.cancelButton, 8, 9, 2, 3)

        self.add(self.table)
        self.show_all()

    def on_checkButtonToggled(self, widget):
        """
        When toggled signal is received (from checkbox, a checkButton object),
        this will set the boolean to staff login.
        """
        self.staff_login = self.checkbox.get_activate_signal()

    def do_quit(self, widget):
        """
        Quit the whole program.
        """
        gtk.main_quit()

    def do_login(self, widget):
        """
        When signal clicked is received from loginButton or "enter" is pressed
        within passwdInput field, will do the login.

        This login will first check the staff_login boolean and then passed to
        corresponding user interface.
        """
        username = self.usernameInput.get_text()
        password = self.passwdInput.get_text()
        import logincontrol
        if not logincontrol.LoginControl().control(username, password,
                self.staff_login):
            pwinc = gtk.Label("Username or password incorrect")
            dialog = gtk.Dialog("Error!", None, gtk.DIALOG_MODAL,
                    (gtk.STOCK_OK, gtk.RESPONSE_ACCEPT))
            dialog.vbox.pack_start(pwinc)
            pwinc.show()
            response = dialog.run()
            dialog.destroy()
        else:
            self.hide_on_delete()
            
    def do_customer_register(self, widget):
        """
        Directly opens the register UI.
        """
        from modules.register import RegisterUI
        RegisterUI()

if __name__ == '__main__':
    win = LoginWindow()
    gtk.main()
