#coding=utf-8
#
# File:     parser.py
# Author:   Luxing Huang 100113287
# Date:     2014/03/12
# Purpose:  This module is only responsible for parsing and initializing
#           configurations stored in shop.cfg
#

import ConfigParser
import os.path

config_file = "shop.cfg"
dbpath = "dvdshop.db"

class Parser(object):
    """
    Configuration parser and initializer for the dvd shop.

    We have the following method:
        __init__() initializing the shop.cfg file if not found, and write the
                  defaults.
        get_shop_name() returns the shop name in a str.
        get_profit_rate()   returns the rate of profit in each DVD.
        get_deposit_rate()  returns the deposit rate when a DVD is lent out.
        get_punish_rate()   returns the overdue charge in percentage per day.
        get_db_path()   returns the database path.

    """
    def __init__(self):
        """
        Checking if the config file exists, otherwise write all the default
        values into the config file.
        """
        if not os.path.exists(config_file):
            self.__write_defaults()

    def __write_defaults(self):
        cp = ConfigParser.RawConfigParser()
        cp.add_section("shop")
        cp.set("shop", "name", "DVD Shop")
        cp.set("shop", "database", "dvdshop.db")
        cp.add_section("rates")
        cp.set("rates", "profit", "0.2")
        cp.set("rates", "deposit", "1.1")
        cp.set("rates", "punish", "0.05")

        with open(config_file, 'w') as f:
            cp.write(f)

    def get_shop_name(self):
        """
        Fetch shop's name
        """
        cp = ConfigParser.ConfigParser()
        cp.read(config_file)
        return cp.get('shop', 'name')

    def get_profit_rate(self):
        """
        Fetch profit rate for 7-day lease of 1 DVD
        """
        cp = ConfigParser.RawConfigParser()
        cp.read(config_file)
        return cp.getfloat('rates', 'profit')

    def get_deposit_rate(self):
        """
        Fetch deposit rate
        """
        cp = ConfigParser.RawConfigParser()
        cp.read(config_file)
        return cp.getfloat('rates', 'deposit')

    def get_punish_rate(self):
        """
        Fetch the default punish rate per day.
        """
        cp = ConfigParser.RawConfigParser()
        cp.read(config_file)
        return cp.getfloat('rate', 'punish')

    def get_db_path(self):
        cp = ConfigParser.ConfigParser()
        cp.read(config_file)
        return cp.get('shop', 'database')
