#coding=utf-8
#
# File:     register.py
# Author:   Luxing Huang 100113287
# Date:     2013/03/13
# Purpose:  This is the registration interface for customer.
#
import gtk

class RegisterUI(gtk.Window):
    """
    Registration Interface
    """
    def __init__(self):
        gtk.Window.__init__(self)
        self.set_position(gtk.WIN_POS_CENTER)
        self.set_title("Registration")
        self.set_size_request(600, 300)
        self.connect("destroy", self.on_destroy)
        self.set_resizable(False)

        # Layout
        table = gtk.Table()

        # Labels
        self.usernameLabel = gtk.Label("Username:")
        self.pass1Label =    gtk.Label("Password:")
        self.pass2Label =    gtk.Label("Repeat  :")
        self.addressLabel =  gtk.Label("Address :")
        self.phoneLabel =    gtk.Label("Mobile   :")
        self.emailLabel =    gtk.Label("Email    :")

        # Text Entries
        self.usernameEntry = gtk.Entry()
        self.usernameEntry.set_width_chars(50)
        self.pass1Entry = gtk.Entry()
        self.pass1Entry.set_visibility(False)
        self.pass2Entry = gtk.Entry()
        self.pass2Entry.set_visibility(False)
        self.addressEntry = gtk.Entry()
        self.phoneEntry = gtk.Entry()
        self.emailEntry = gtk.Entry()

        # Buttons
        self.regButton = gtk.Button("Register")
        self.regButton.set_size_request(70, 30)
        self.cancelButton = gtk.Button("Cancel")
        self.cancelButton.set_size_request(70, 30)
        self.resetButton = gtk.Button("Reset")
        self.resetButton.set_size_request(70, 30)

        # Add elements to layouts

        table.attach(self.usernameLabel, 0, 1, 0, 1)
        table.attach(self.usernameEntry, 1, 2, 0, 1)
        table.attach(self.pass1Label, 0, 1, 1, 2)
        table.attach(self.pass1Entry, 1, 2, 1, 2)
        table.attach(self.pass2Label, 0, 1, 2, 3)
        table.attach(self.pass2Entry, 1, 2, 2, 3)
        table.attach(self.addressLabel, 0, 1, 3, 4)
        table.attach(self.addressEntry, 1, 2, 3, 4)
        table.attach(self.phoneLabel, 0, 1, 4, 5)
        table.attach(self.phoneEntry, 1, 2, 4, 5)
        table.attach(self.emailLabel, 0, 1, 5, 6)
        table.attach(self.emailEntry, 1, 2, 5, 6)
        table.attach(self.regButton, 0, 3, 6, 7)
        table.attach(self.cancelButton, 0, 3, 6, 7)
        table.attach(self.resetButton, 0, 3, 7, 8)

        # Signals
        self.regButton.connect("clicked", self.do_register)
        self.resetButton.connect("clicked", self.reset_entries)
        self.cancelButton.connect("clicked", self.on_destroy)
        self.phoneEntry.connect("activate", self.do_register)

        self.add(table)

        # Misc
        self.show_all()

    def on_destroy(self, widget):
        """
        Close itself.
        """
        self.destroy()

    def reset_entries(self, widget):
        """
        Set the textfields to none, allowing the user to re-type everything.
        """
        self.usernameEntry.set_text("")
        self.pass1Entry.set_text("")
        self.pass2Entry.set_text("")
        self.addressEntry.set_text("")
        self.phoneEntry.set_text("")
        self.emailEntry.set_text("")

    def do_register(self, widget):
        """
        Registration process, connection with the usercontroller.

        It will also check whether user exists or not.
        """
        username = self.usernameEntry.get_text()
        password = self.pass1Entry.get_text()
        address = self.addressEntry.get_text()
        phoneNumber = self.phoneEntry.get_text()
        email = self.emailEntry.get_text()

        if username == "" or password == "" or address == "" or \
                phoneNumber == "" or email == "":
            dialog = gtk.Dialog("Error!", self, gtk.DIALOG_MODAL, 
                    (gtk.STOCK_OK, gtk.RESPONSE_ACCEPT |
                    gtk.DIALOG_DESTROY_WITH_PARENT))
            reg_suc = gtk.Label("Please fill all information required.")
            dialog.vbox.pack_start(reg_suc)
            reg_suc.show()
            dialog.run()
            dialog.destroy()
            return

        if not self.pass1Entry.get_text() == self.pass2Entry.get_text():
            dialog = gtk.Dialog("Error!", self, gtk.DIALOG_MODAL, 
                    (gtk.STOCK_OK, gtk.RESPONSE_ACCEPT |
                        gtk.DIALOG_DESTROY_WITH_PARENT))
            reg_suc = gtk.Label("Passwords didn't match!")
            dialog.vbox.pack_start(reg_suc)
            reg_suc.show()
            dialog.run()
            dialog.destroy()
            return

        from usercontroller import UserController
        db = UserController()
        ret = db.add_user(username, password, address, phoneNumber, email)

        if ret == True:
            from logger import Logger
            log = Logger()
            log.log_add_user(username, 'userdb', 'None', 0)

            dialog = gtk.Dialog("Information", self,
                    gtk.DIALOG_MODAL, (gtk.STOCK_OK, gtk.RESPONSE_ACCEPT |
                        gtk.DIALOG_DESTROY_WITH_PARENT))
            reg_suc = gtk.Label("Registration Successful!")
            dialog.vbox.pack_start(reg_suc)
            reg_suc.show()
            dialog.run()
            dialog.destroy()
            self.on_destroy(None)

        else:
            dialog = gtk.Dialog("Error", self,
                    gtk.DIALOG_MODAL, (gtk.STOCK_OK, gtk.RESPONSE_ACCEPT |
                        gtk.DIALOG_DESTROY_WITH_PARENT))
            reg_suc = gtk.Label("User Exists!")
            dialog.vbox.pack_start(reg_suc)
            reg_suc.show()
            dialog.run()
            dialog.destroy()
