#coding=utf-8
#
# COMP 2663
# Project:  DVD Shop
# Author:   Luxing Huang    100113287
# Date:     2014/03/09
# File:     shopdatabase.py
# Purpose:  This is the database using sqlite3 on DVD list and user management.

import sqlite3
import parser

ps = parser.Parser()
dbpath = ps.get_db_path()

class DatabaseController(object):
    """
    This class initializes the entire database, and it's inherited by other
    controller who will be using database.
    """
    def __init__(self):
        """
        Initialize the database, check if the database is correctly built or
        not
        """
        if not self.__check_db():
            self.__db_init()

    def __check_db(self):
        """
        Private helper method that can check if the database is correctly
        initiated, with correct tables

        It returns False on table dvdlist not found. This means the database
        needs to be initialized. Otherwise return true, no need for
        initialization.
        """
        conn = sqlite3.connect(dbpath)
        try:
            c = conn.cursor()
            test = c.execute("SELECT * FROM dvdlist")
        except sqlite3.OperationalError:
            conn.close()
            return False
        else:
            conn.close()
            return True

    def __db_init(self):
        """
        Initializing the database with dvdlist table, userdb table and finally
        staffdb
        """
        conn = sqlite3.connect(dbpath)
        c = conn.cursor()
        c.executescript("""
        CREATE TABLE IF NOT EXISTS dvdlist (
            title text, 
            barcode integer, 
            added_date text, 
            out numeric,
            borrower text,
            price real
        );

        CREATE TABLE IF NOT EXISTS userdb (
            username text, 
            password text,
            address text,
            phone text,
            email text,
            borrowed integer
        );

        CREATE TABLE IF NOT EXISTS staffdb (
            username text,
            password text,
            address text,
            phone text,
            email text,
            priv integer
        );

        CREATE TABLE IF NOT EXISTS paylog (
            amount real,
            operator text
        );
        """)
        conn.commit()
        conn.close()
