from shopdatabase import DatabaseController
from parser import Parser
import sqlite3

ps = Parser()
dbpath = ps.get_db_path()

class StockController(DatabaseController):
    """
    delete_dvd(barcode)
    stock_dvd(dvdname, barcode, price) <==> add_dvd(...)
    search_dvd(dvdname)
    list_all_dvd()
    set_dvd(barcode, **kwargs) --> Please see the specific doc below.
    """
    def __init__(self):
        DatabaseController.__init__(self)

    def delete_dvd(self, barcode):
        """
        Deleting a DVD based on its barcode

        None returned.
        """
        conn = sqlite3.connect(dbpath)
        c = conn.cursor()
        c.execute("DELETE FROM dvdlist WHERE barcode=(?)",(barcode,))
        conn.commit()
        conn.close()

    def stock_dvd(self, dvdname, barcode, price):
        """
        Adding DVD into the database, allowing same title with different
        barcode.
        """
        conn = sqlite3.connect(dbpath)
        c = conn.cursor()
        test = c.execute("SELECT * FROM dvdlist WHERE barcode = (?)",
                (barcode,))
        try:
            test.next()
        except StopIteration:
            import datetime
            date = datetime.datetime.now()
            c.execute("INSERT INTO dvdlist VALUES (?, ?, ?, 0, null, ?)",
                    (dvdname, barcode, date, price))
            conn.commit()
            conn.close()
            return True
        else:
            conn.close()
            return False

    def search_dvd(self, dvdname):
        """
        Fetch DVD will lookup for DVD name and return their barcodes as a list.
        """
        conn = sqlite3.connect(dbpath)
        c = conn.cursor()
        fetch = c.execute("SELECT * FROM dvdlist WHERE title=(?)", (dvdname,))
        selected = fetch.fetchall()
        conn.close()
        return selected

    def list_all_dvd(self):
        """
        Arguments: None

        Returns a list containing 1 or more tuples that descripts the DVD
        information in the database
        """
        conn = sqlite3.connect(dbpath)
        c = conn.cursor()
        elem = c.execute("SELECT * FROM dvdlist")
        dvds = elem.fetchall()
        conn.close()
        return dvds

    def set_dvd(self, barcode, **kwargs):
        """
        Change the information on a specific DVD, given by a barcode.

        Fields are: title='string', price=float, borrower=int, out=0/1
        """
        conn = sqlite3.connect(dbpath)
        c = conn.cursor()
        if 'title' in kwargs.keys() and kwargs['title'] is not None:
            c.execute("UPDATE dvdlist SET title=(?) WHERE barcode=(?)",
                    (title, barcode))

        if 'price' in kwargs.keys() and kwargs['price'] is not None:
            c.execute("UPDATE dvdlist SET price=(?) WHERE barcode=(?)",
                    (price, barcode))

        if 'borrower' in kwargs.keys() and kwargs['borrower'] is not None:
            c.execute("UPDATE dvdlist SET borrower=(?) WHERE barcode=(?)",
                    (borrower, barcode))

        if 'out' in kwargs.keys() and kwargs['out'] is not None:
            c.execute("UPDATE dvdlist SET out=(?) WHERE barcode=(?)",
                    (out, barcode))

        conn.commit()
        conn.close()

    def get_dvd_info(self, barcode, field="*"):
        """
        Return all the information related DVD on its corresponding barcode.
        """
        conn = sqlite3.connect(dbpath)
        c = conn.cursor()
        elem = c.execute("SELECT " + field + " FROM dvdlist WHERE barcode=(?)",
                (barcode,))
        try:
            dvd = elem.fetchone()
        except StopIteration:
            conn.close()
            return None
        else:
            conn.close()
            return dvd

    # Aliases
    add_dvd = stock_dvd
